@@ -2012,7 +2012,15 @@ Imlib_load_image(ImlibData * id, char *file)
     return NULL;
   if (id->cache.on_image)
     if ((im = find_image(id, file)))
-      return im;
+      {
+        if (im->rgb_width == 0 || im->rgb_height == 0)
+          {
+            Imlib_destroy_image(id, im);
+            return NULL;
+          }
+        else
+          return im;
+      }
   if (!strcmp(file,"-")) {
   	p = stdin;
   }
@@ -2226,6 +2234,15 @@ Imlib_load_image(ImlibData * id, char *file)
       fprintf(stderr, "IMLIB ERROR: Cannot load image: %s\nAll fallbacks failed.\n", fil);
       return NULL;
     }
+    
+  if (!w || !h)
+    {
+      fprintf(stderr, "IMLIB ERROR: zero image\n" );
+      if(data)
+        free(data);
+      return NULL;
+    }
+    
   im = (ImlibImage *) malloc(sizeof(ImlibImage));
   if (!im)
     {
