@@ -17,6 +17,10 @@ loader_png (FILE * f, int *w, int *h, int *t)
 
   /* Init PNG Reader */
   transp = 0;
+#if 0
+   /* 
+    *Corrup[ting images on load is better than no image at all
+    */
   if (!strcmp("1.0.2", png_libpng_ver))
     {
       fprintf(stderr, "WARNING! You have libpng 1.0.2\n"
@@ -24,6 +28,7 @@ loader_png (FILE * f, int *w, int *h, int *t)
 	      "please use 1.0.1. PNG support is disabled.\n");
       return NULL;
     }
+#endif
   png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
   if (!png_ptr)
     return NULL;
@@ -404,7 +409,7 @@ _loader_alpha_png (FILE * f, int *w, int *h, int *t, unsigned char **alpha)
 {
 	png_structp         png_ptr;
 	png_infop           info_ptr;
-	unsigned char      *data, *ptr, **lines, *ptr2, r, g, b, a, *aptr;
+	unsigned char      *data, *ptr, **lines, *ptr2, *aptr;
 	int                 i, x, y, transp, bit_depth, color_type, interlace_type;
 	png_uint_32         ww, hh;
 	
@@ -496,6 +501,8 @@ _loader_alpha_png (FILE * f, int *w, int *h, int *t, unsigned char **alpha)
 	aptr = *alpha;
 	if (color_type == PNG_COLOR_TYPE_GRAY_ALPHA)
 	{
+		unsigned char r;
+
 		for (y = 0; y < *h; y++)
 		{
 			ptr2 = lines[y];
@@ -511,6 +518,8 @@ _loader_alpha_png (FILE * f, int *w, int *h, int *t, unsigned char **alpha)
 	}
 	else if (color_type == PNG_COLOR_TYPE_GRAY)
 	{
+		unsigned char r;
+
 		for (y = 0; y < *h; y++)
 		{
 			ptr2 = lines[y];
