@@ -2,6 +2,7 @@
 #ifndef GNOMELOCALEDIR
 #define GNOMELOCALEDIR "/usr/share/locale"
 #endif
+#include <locale.h>
 #ifdef ENABLE_NLS
 #    include <libintl.h>
 #    define _(String) gettext (String)
@@ -32,7 +33,6 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
-#include <locale.h>
 
 GtkWidget          *note = NULL;
 GtkWidget          *pg1 = NULL;
@@ -202,16 +202,47 @@ toggle_onoff(GtkWidget * widget, char *value)
       *value = 1;
 }
 
+void
+redraw_palette(void)
+{
+  int                 i, j;
+  unsigned char      *ptr;
+  GdkGC              *gc;
+  GdkColor            clr;
+  GdkColormap        *cmap;
+  
+  if (color.pmap)
+    gdk_imlib_free_pixmap(color.pmap);
+  ptr = color.pal_im->rgb_data;
+  for (i = 0; i < 256; i++)
+    {
+      for (j = 0; j < 3; j++)
+	{
+	  *ptr++ = color.color[i][j];
+	}
+    }
+  gdk_imlib_render(color.pal_im, 128, 128);
+  color.pmap = gdk_imlib_move_image(color.pal_im);
+  gdk_window_set_back_pixmap(color.pal_widget->window, color.pmap, FALSE);
+  gdk_imlib_changed_image(color.pal_im);
+  gdk_window_clear(color.pal_widget->window);
+  gc = gdk_gc_new(color.pal_widget->window);
+  cmap = gdk_colormap_get_system();
+  gdk_color_white(cmap, &clr);
+  gdk_gc_set_foreground(gc, &clr);
+  gdk_gc_set_function(gc, GDK_XOR);
+  gdk_draw_rectangle(color.pal_widget->window, gc, 0,
+		     (color.cur_color % 16) * 8, (color.cur_color & 0xf0) / 2,
+		     7, 7);
+  gdk_gc_destroy(gc);
+  gdk_flush();
+}
+
 void
 change_col(GtkWidget * widget, GtkWidget * w)
 {
    gdouble             col[4];
-   int                 i, j;
    int                 r, g, b;
-   unsigned char      *ptr;
-   GdkGC              *gc;
-   GdkColor            clr;
-   GdkColormap        *cmap;
 
    gtk_color_selection_get_color(GTK_COLOR_SELECTION(w), col);
    if (color.cur_color >= 0)
@@ -227,32 +258,8 @@ change_col(GtkWidget * widget, GtkWidget * w)
 	     color.color[color.cur_color][1] = g;
 	     color.color[color.cur_color][2] = b;
 	  }
-	if (color.pmap)
-	   gdk_imlib_free_pixmap(color.pmap);
-	ptr = color.pal_im->rgb_data;
-	for (i = 0; i < 256; i++)
-	  {
-	     for (j = 0; j < 3; j++)
-	       {
-		  *ptr++ = color.color[i][j];
-	       }
-	  }
-	gdk_imlib_render(color.pal_im, 128, 128);
-	color.pmap = gdk_imlib_move_image(color.pal_im);
-	gdk_window_set_back_pixmap(color.pal_widget->window, color.pmap, FALSE);
-	gdk_imlib_changed_image(color.pal_im);
-	gdk_window_clear(color.pal_widget->window);
-	gc = gdk_gc_new(color.pal_widget->window);
-	cmap = gdk_colormap_get_system();
-	gdk_color_white(cmap, &clr);
-	gdk_gc_set_foreground(gc, &clr);
-	gdk_gc_set_function(gc, GDK_XOR);
-	gdk_draw_rectangle(color.pal_widget->window, gc, 0,
-		     (color.cur_color % 16) * 8, (color.cur_color & 0xf0) / 2,
-			   7, 7);
-	gdk_gc_destroy(gc);
-	gdk_flush();
      }
+  redraw_palette();
 }
 
 void
@@ -399,7 +406,7 @@ pal_load(GtkWidget * widget, GtkWidget * w)
 {
    if (conf.palettefile)
       gtk_file_selection_set_filename(GTK_FILE_SELECTION(w), conf.palettefile);
-   showhide(NULL, w);
+  showhide(NULL, w);
 }
 
 void
@@ -415,7 +422,7 @@ void
 load_pal(char *f)
 {
    FILE               *ff;
-   int                 r, g, b;
+   int                 r, g, b, i;
    char                s[1024];
 
   setlocale(LC_ALL, "C");
@@ -447,6 +454,12 @@ load_pal(char *f)
 	  }
 	fclose(ff);
      }
+  for (i = color.number; i < 256; i++)
+    {
+      color.color[i][0] = 0;
+      color.color[i][1] = 0;
+      color.color[i][2] = 0;
+     }
   setlocale(LC_ALL, "");
 }
 
@@ -797,27 +810,24 @@ read_file(GtkWidget * widget, GtkWidget * w)
 	gtk_entry_set_text(GTK_ENTRY(pal_text), s);
      }
    else
-     {
-	load_pal(s);
-	gtk_entry_set_text(GTK_ENTRY(pal_text), s);
+    {
+      load_pal(s);
+      gtk_entry_set_text(GTK_ENTRY(pal_text), s);
+      redraw_palette();
      }
 }
 
 void
 mod_int(GtkWidget * widget, struct _vint *v)
 {
-   *(v->value) = (int)GTK_ADJUSTMENT(v->adj)->value;
-   update_test();
+  *(v->value) = (int)GTK_ADJUSTMENT(v->adj)->value;
+  update_test();
 }
 
 void
 reset_int(GtkWidget * widget, struct _vint *v)
 {
-   *(v->value) = 256;
-   GTK_ADJUSTMENT(v->adj)->value = (gfloat) * (v->value);
-   gtk_range_set_adjustment(GTK_RANGE(v->range), GTK_ADJUSTMENT(v->adj));
-   gtk_range_slider_update(GTK_RANGE(v->range));
-   update_test();
+   gtk_adjustment_set_value(GTK_ADJUSTMENT(v->adj), 256.0);
 }
 
 void
@@ -931,9 +941,10 @@ add_sliders(GtkWidget * w)
    gtk_widget_show(i2);
    i3 = gtk_pixmap_new(contrast_pmap, contrast_mask);
    gtk_widget_show(i3);
+
    frame = gtk_aspect_frame_new(_("Base Levels"), 0.5, 0.5, 0.0, TRUE);
    gtk_box_pack_start(GTK_BOX(box1), frame, FALSE, FALSE, 4);
-   gtk_widget_show(frame);
+   /* gtk_widget_show(frame); */
    box = gtk_vbox_new(TRUE, 0);
    gtk_container_add(GTK_CONTAINER(frame), box);
    gtk_widget_show(box);
@@ -992,6 +1003,21 @@ add_sliders(GtkWidget * w)
    add_test(box0);
 }
 
+void
+inbuilt_pal(GtkWidget *widget, char *p)
+{
+  char s[4096], *def;
+  int i;
+  
+  def = gdk_imlib_get_sysconfig();
+  for (i = strlen(def) - 1; ((i >= 0) && (def[i] != '/')); def[i--] = 0);
+  g_snprintf(s, sizeof(s), "%s%s", def, p);
+  free(def);
+  load_pal(s);
+  gtk_entry_set_text(GTK_ENTRY(pal_text), s);
+  redraw_palette();
+}
+
 void
 add_palsel(GtkWidget * w, char *title, char **value)
 {
@@ -1002,6 +1028,25 @@ add_palsel(GtkWidget * w, char *title, char **value)
    gtk_box_pack_start(GTK_BOX(w), box, TRUE, TRUE, 0);
    gtk_widget_show(box);
 
+   box2 = gtk_hbox_new(TRUE, 0);
+   gtk_box_pack_start(GTK_BOX(box), box2, TRUE, TRUE, 0);
+   gtk_widget_show(box2);
+  
+   button = gtk_button_new_with_label(_("Small colour set"));
+   gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 2);
+   gtk_widget_show(button);
+   gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(inbuilt_pal), "im_palette-tiny.pal");
+
+   button = gtk_button_new_with_label(_("Medium colour set"));
+   gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 2);
+   gtk_widget_show(button);
+   gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(inbuilt_pal), "im_palette-small.pal");
+
+   button = gtk_button_new_with_label(_("Large colour set"));
+   gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 2);
+   gtk_widget_show(button);
+   gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(inbuilt_pal), "im_palette.pal");
+  
    box2 = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(box), box2, TRUE, TRUE, 0);
    gtk_widget_show(box2);
@@ -1016,12 +1061,12 @@ add_palsel(GtkWidget * w, char *title, char **value)
 		      "clicked", GTK_SIGNAL_FUNC(read_file), filesel);
 
    button = gtk_button_new_with_label(_("Select"));
-   gtk_box_pack_start(GTK_BOX(box2), button, FALSE, FALSE, 0);
+   gtk_box_pack_start(GTK_BOX(box2), button, FALSE, FALSE, 2);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(pal_load), filesel);
 
    text = gtk_entry_new_with_max_length(1024);
-   gtk_box_pack_start(GTK_BOX(box2), text, TRUE, TRUE, 0);
+   gtk_box_pack_start(GTK_BOX(box2), text, TRUE, TRUE, 2);
    gtk_widget_show(text);
    if (*value)
       gtk_entry_set_text(GTK_ENTRY(text), *value);
@@ -1031,7 +1076,7 @@ add_palsel(GtkWidget * w, char *title, char **value)
    pal_text = text;
 
    button = gtk_button_new_with_label(_("Save"));
-   gtk_box_pack_start(GTK_BOX(box2), button, FALSE, FALSE, 0);
+   gtk_box_pack_start(GTK_BOX(box2), button, FALSE, FALSE, 2);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(pal_save), filesel);
 
@@ -1058,31 +1103,31 @@ add_palsel(GtkWidget * w, char *title, char **value)
    gtk_signal_connect(GTK_OBJECT(pals), "button_press_event", GTK_SIGNAL_FUNC(pal_press), NULL);
    gtk_widget_set_events(pals, GDK_BUTTON_PRESS_MASK);
 
-   box3 = gtk_hbox_new(FALSE, 0);
-   gtk_box_pack_start(GTK_BOX(box), box3, TRUE, TRUE, 0);
+   box3 = gtk_hbox_new(TRUE, 2);
+   gtk_box_pack_start(GTK_BOX(box), box3, TRUE, TRUE, 2);
    gtk_widget_show(box3);
 
    button = gtk_button_new_with_label(_("Copy To"));
-   gtk_box_pack_start(GTK_BOX(box3), button, TRUE, TRUE, 0);
+   gtk_box_pack_start(GTK_BOX(box3), button, TRUE, TRUE, 2);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(col_copy), NULL);
 
    button = gtk_button_new_with_label(_("Spread To"));
-   gtk_box_pack_start(GTK_BOX(box3), button, TRUE, TRUE, 0);
+   gtk_box_pack_start(GTK_BOX(box3), button, TRUE, TRUE, 2);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(col_spread), NULL);
 
-   box3 = gtk_hbox_new(FALSE, 0);
-   gtk_box_pack_start(GTK_BOX(box), box3, TRUE, TRUE, 0);
+   box3 = gtk_hbox_new(TRUE, 2);
+   gtk_box_pack_start(GTK_BOX(box), box3, TRUE, TRUE, 2);
    gtk_widget_show(box3);
 
    button = gtk_button_new_with_label(_("Set Last"));
-   gtk_box_pack_start(GTK_BOX(box3), button, TRUE, TRUE, 0);
+   gtk_box_pack_start(GTK_BOX(box3), button, TRUE, TRUE, 2);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(col_setlast), NULL);
 
    button = gtk_button_new_with_label(_("Delete"));
-   gtk_box_pack_start(GTK_BOX(box3), button, TRUE, TRUE, 0);
+   gtk_box_pack_start(GTK_BOX(box3), button, TRUE, TRUE, 2);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(col_del), NULL);
 
@@ -1290,7 +1335,7 @@ add_onoff_visual(GtkWidget * w, char *title)
    gtk_widget_show(box2);
 
    scrolled_win = gtk_scrolled_window_new (NULL, NULL);
-   clist = gtk_clist_new_with_titles(3, titles);
+   clist = gtk_clist_new_with_titles(3, (char **)titles);
    gtk_box_pack_start(GTK_BOX(w), scrolled_win, TRUE, TRUE, 0);
    gtk_container_add (GTK_CONTAINER (scrolled_win), clist);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_win), GTK_POLICY_ALWAYS, GTK_POLICY_AUTOMATIC);
@@ -1317,19 +1362,27 @@ add_loadsave()
 {
    GtkWidget          *button, *box2;
 
-   box2 = gtk_hbox_new(FALSE, 0);
+   box2 = gtk_hbox_new(TRUE, 0);
    gtk_box_pack_start(GTK_BOX(list), box2, FALSE, FALSE, 8);
    gtk_widget_show(box2);
 
    button = gtk_button_new_with_label(_("Save User Config"));
-   gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);
+   gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 32);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(save_user), NULL);
 
-   button = gtk_button_new_with_label(_("Save System Config"));
-   gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);
+  if (getuid() == 0)
+    {
+      button = gtk_button_new_with_label(_("Save System Config"));
+      gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 32);
+      gtk_widget_show(button);
+      gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(save_sys), NULL);
+    }
+
+   button = gtk_button_new_with_label(_("Close"));
+   gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 32);
    gtk_widget_show(button);
-   gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(save_sys), NULL);
+   gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(kill_win), NULL);
 }
 
 void
